@@ -1,12 +1,12 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "prodigy" "1.0.0"
+(define-package "prodigy" "20250401.1948"
   "Manage external services."
   '((s     "1.8.0")
     (dash  "2.4.0")
     (f     "0.14.0")
     (emacs "27.1"))
   :url "http://github.com/rejeep/prodigy.el"
-  :commit "b67e4112eaca7ef7c8596c4728e9bc53418982d7"
-  :revdesc "b67e4112eaca"
+  :commit "7bd89fdd544209afb28d6abe828728ad62257617"
+  :revdesc "7bd89fdd5442"
   :authors '(("Johan Andersson" . "johan.rejeep@gmail.com"))
   :maintainers '(("Johan Andersson" . "johan.rejeep@gmail.com")))
