@@ -1,11 +1,11 @@
 ;;; prodigy.el --- Manage external services -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2013-2022 Johan Andersson
+;; Copyright (C) 2013-2025 Johan Andersson
 
 ;; Author: Johan Andersson <johan.rejeep@gmail.com>
 ;; Maintainer: Johan Andersson <johan.rejeep@gmail.com>
-;; Package-Version: 1.0.0
-;; Package-Revision: b67e4112eaca
+;; Package-Version: 20250401.1948
+;; Package-Revision: 7bd89fdd5442
 ;; URL: http://github.com/rejeep/prodigy.el
 ;; Package-Requires: ((s "1.8.0") (dash "2.4.0") (f "0.14.0") (emacs "27.1"))
 
@@ -219,9 +219,10 @@ The list is a property list with the following properties:
   environment variable and second item is the value of the variable.
 
 `url'
-  Single URL or list of URLs to use for browsing.  If single URL is
-  specified, use that.  If a list of URLs are specified, ask for what
-  URL to browse.
+  Single URL or list of URLs to use for browsing.  If single
+  URL is a string, use that.  If single URL is a function,
+  evaluate the function and use the result.  If a list of URLs
+  is specified, ask for what URL to browse.
 
 `kill-process-buffer-on-stop'
   Kill associated process buffer when process stops.
@@ -539,7 +540,10 @@ SERVICE tag that has and return that."
 
 If SERVICE's `url' exists, use that.  If not, find the first SERVICE
 tag that has and return that."
-  (prodigy-service-or-first-tag-with service :url))
+  (let ((url (prodigy-service-or-first-tag-with service :url)))
+    (if (functionp url)
+        (prodigy-callback-with-plist url service)
+      url)))
 
 (defun prodigy-service-on-output (service)
   "Return SERVICE and its tags on-output functions as list.
@@ -648,7 +652,7 @@ All windows from all frames are considered."
   "Return SERVICE PROPERTY or tag with PROPERTY.
 
 If SERVICE has PROPERTY, return the value of that property.  Note
-that '(:foo nil) means that the list has the property :foo.  If
+that (:foo nil) means that the list has the property :foo.  If
 SERVICE does not have property, find the first SERVICE tag that
 has that property and return its value."
   (if (plist-member service property)
@@ -824,7 +828,16 @@ If SERVICE defines several URLs, ask the user which one is
 preferred."
   (-when-let (url (prodigy-url service))
     (if (listp url)
-        (prodigy-completing-read "URL: " url)
+        (prodigy-completing-read
+         "URL: "
+         (lambda (string predicate action)
+           (if (eq action 'metadata)
+               ;; don't sort candidates so the user can learn their
+               ;; positions by heart:
+               `(metadata (display-sort-function . ,#'identity)
+                          (cycle-sort-function . ,#'identity))
+             (complete-with-action
+              action url string predicate))))
       url)))
 
 (defun prodigy-discover-initialize ()
@@ -851,7 +864,7 @@ preferred."
 (defun prodigy-move-until (direction callback)
   "Move in DIRECTION until while CALLBACK return false.
 
-DIRECTION is either 'up or 'down."
+DIRECTION is either `up' or `down'."
   (let ((pos (line-beginning-position))
         (found
          (catch 'break
@@ -1136,16 +1149,25 @@ put in stopped status."
       (when (process-live-p process)
         (prodigy-set-status service 'stopping)
         (let ((stop-signal (prodigy-service-stop-signal service)))
-          (cond ((eq stop-signal 'int)
-                 (interrupt-process process))
-                ((or force (eq stop-signal 'kill))
-                 (kill-process process))
-                ((eq stop-signal 'quit)
-                 (quit-process process))
-                ((eq stop-signal 'stop)
-                 (stop-process process))
-                (t
-                 (signal-process process (or stop-signal 'int)))))
+          (if (plist-get service :sudo)
+              (prodigy-start-sudo-process
+               "*prodigy-sudo-kill*" nil
+               "kill"
+               (concat "-" (cond
+                            ((null stop-signal) "INT")
+                            ((symbolp stop-signal) (upcase (symbol-name stop-signal)))
+                            (force "KILL")))
+               (concat "-" (number-to-string (process-id process))))
+            (cond ((eq stop-signal 'int)
+                   (interrupt-process process))
+                  ((or force (eq stop-signal 'kill))
+                   (kill-process process))
+                  ((eq stop-signal 'quit)
+                   (quit-process process))
+                  ((eq stop-signal 'stop)
+                   (stop-process process))
+                  (t
+                   (signal-process process (or stop-signal 'int))))))
         (let ((tryout 0))
           (prodigy-every 1
               (lambda (next)
@@ -1388,8 +1410,7 @@ Customize `prodigy-file-manager' to choose your favorite file manager."
 ;;;; View mode functions
 
 (defun prodigy-strip-ctrl-m (output)
-  "Strip 
- line endings from OUTPUT."
+  "Strip ^M line endings from OUTPUT."
   (s-replace "
 " "" output))
 
@@ -1531,12 +1552,12 @@ beginning of the line."
 (defun prodigy ()
   "Manage external services from within Emacs."
   (interactive)
-  (let ((buffer-p (prodigy-buffer))
-        (buffer (get-buffer-create prodigy-buffer-name)))
-    (pop-to-buffer buffer)
-    (unless buffer-p
-      (prodigy-mode))
-    (prodigy-start-status-check-timer)))
+  (let ((buffer (get-buffer-create prodigy-buffer-name)))
+    (with-current-buffer buffer
+      (unless (eq major-mode 'prodigy-mode)
+        (prodigy-mode))
+      (prodigy-start-status-check-timer))
+    (pop-to-buffer buffer)))
 
 (provide 'prodigy)
 
